
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Random;

/**
 * Composite Numbers
 *
 * For the UMD High School Programming Contest, 2009.
 * @author Chau-Wen Tseng
 */
public class Composites {
	// static int testStart = 1000000000;
	// static int testStop = 1000100000;
	static int testStart = 1;
	static int testStop = 10000000;

	public static void main(String[] args) throws Exception {
		int seqLen = 0;
		int seqStart = 0;
		int seqEnd = 0;
		int start = 0;
		long time;

		// System.out.println(Integer.MAX_VALUE);

		time = System.currentTimeMillis();
		sieveAlg1();
		time = System.currentTimeMillis() - time;
		System.out.println("Sieve Time = "+(time/1000));
		
		time = System.currentTimeMillis();
		simpleAlg();
		time = System.currentTimeMillis() - time;
		System.out.println("Simple Time = "+(time/1000));

		time = System.currentTimeMillis();
		inlineAlg();
		time = System.currentTimeMillis() - time;
		System.out.println("Inline Time = "+(time/1000));
	}

	public static void simpleAlg () {
		int seqLen = 0;
		int seqStart = 0;
		int seqEnd = 0;
		int start = 0;		

		for (int i = testStart; i<= testStop; i++) {
			if (isPrime(i)) {
				// sequence is from start to i-1
				int len = (i-1)-start;
				if ((start != 0) && (len > seqLen)) {
					seqStart = start;
					seqEnd = i-1;
					// System.out.println(seqStart + " to " + seqEnd  + ": " + len);
					seqLen = len;
					start = 0;
				}
			} else if (start == 0){
				start = i;
			}
		}

		System.out.println(seqStart + " to " + seqEnd + " length = " + seqLen);

	}

	public static void inlineAlg () {
		int seqLen = 0;
		int seqStart = 0;
		int seqEnd = 0;
		int start = 0;		
		boolean prime;
		
		for (int i = testStart; i<= testStop; i++) {
			int limit = (int) Math.sqrt((double) i);

			prime = true;
			int k;
			for ( k = 2; k <= limit; k++ )
			{
				if ( i % k == 0 ) {
					prime = false;
					break;
				}
			}

			// if prime
			if (prime) {
				// sequence is from start to i-1
				int len = (i-1)-start;
				if ((start != 0) && (len > seqLen)) {
					seqStart = start;
					seqEnd = i-1;
					// System.out.println(seqStart + " to " + seqEnd  + ": " + len);
					seqLen = len;
					start = 0;
				}
			} else if (start == 0){
				start = i;
			}
		}
		
		System.out.println(seqStart + " to " + seqEnd + " length = " + seqLen);

	}

	public static void sieveAlg1 () {
		int limit = 1 + (int) Math.sqrt((double) testStop);

		boolean [] flags = new boolean[testStop+1];

		// initialize flags
		for (int i=0; i <= testStop; i++) {
			flags[i] = true;
		}

		// for all divisors from 2 to sqrt(top of range)
		for (int d = 2; d < limit; d++) {
			// if divisor is prime
			if (flags[d]) {
				// remove all multiples of prime d  
				for (int k=d+d; k <= testStop; k+=d) {
					flags[k] = false;
				}
			}
		}

		// check for longest run of composite numbers
		int seqLen = 0;
		int seqStart = 0;
		int seqEnd = 0;
		int start = 0;	
		
		for (int i = testStart; i<= testStop; i++) {
			if (flags[i]) {
				// sequence is from start to i-1
				int len = (i-1)-start;
				if ((start != 0) && (len > seqLen)) {
					seqStart = start;
					seqEnd = i-1;
					// System.out.println(seqStart + " to " + seqEnd  + ": " + len);
					seqLen = len;
					start = 0;
				}
			} else if (start == 0){
				start = i;
			}
		}
		
		System.out.println(seqStart + " to " + seqEnd + " length = " + seqLen);

	}

	public static boolean isPrime(int num) {
		int limit = (int) Math.sqrt((double) num);
		for ( int i = 2; i <= limit; i++ )
		{
			if ( num % i == 0 )
				return false;
		}
		return true;
	}
}
